@@ -4,6 +4,7 @@
 #include <iostream>
 #include "NWeighted.hpp"
 #include "Vertex.hpp"
+#include "VertexRegistration.hpp"
 
 namespace graph_analysis {
 
@@ -17,7 +18,7 @@ class NWeightedVertex : public NWeighted<T,Dim,Vertex>
 public:
     typedef shared_ptr< NWeightedVertex > Ptr;
 
-    NWeightedVertex(const T& weight)
+    NWeightedVertex(const T& weight = T())
         : NWeighted<T,Dim,Vertex>(weight)
     {}
 
@@ -25,10 +26,13 @@ public:
 
     // Get class name
     // \return class name
-    virtual std::string getClassName() const { return "graph_analysis::NWeightedVertex"; }
+    virtual std::string getClassName() const { return
+        "graph_analysis::NWeightedVertex_" + std::string(typeid(T).name()) + "_" + std::to_string(Dim); }
 
 protected:
     virtual Vertex* getClone() const { return new NWeightedVertex<T,Dim>(*this); }
+
+    static const VertexRegistration< NWeightedVertex<T,Dim> >  __attribute__((used)) msRegistration;
 };
 
 } // end namespace graph_analysis
